@@ -31,6 +31,7 @@
 #include "todometadatamodel.h"
 #include "testlib/testlib.h"
 #include "testlib/modeltest.h"
+#include "testlib/modelbuilderbehavior.h"
 
 using namespace Zanshin::Test;
 
@@ -144,8 +145,8 @@ private slots:
         QFETCH(ModelPath::List, itemsToRemove);
 
         // Collect data to ensure we signalled the outside properly
-        QSignalSpy aboutToRemoveSpy(&treeModel, SIGNAL(rowsAboutToBeRemoved(QModelIndex, int, int)));
-        QSignalSpy removeSpy(&treeModel, SIGNAL(rowsRemoved(QModelIndex, int, int)));
+        QSignalSpy aboutToRemoveSpy(&treeModel, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)));
+        QSignalSpy removeSpy(&treeModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
 
         QList<QModelIndex> parents;
         QList<int> rows;
@@ -401,8 +402,8 @@ private slots:
         QFETCH(ModelStructure, insertedStructure);
 
         // Collect data to ensure we signalled the outside properly
-        QSignalSpy aboutToInsertSpy(&treeModel, SIGNAL(rowsAboutToBeInserted(QModelIndex, int, int)));
-        QSignalSpy insertSpy(&treeModel, SIGNAL(rowsInserted(QModelIndex, int, int)));
+        QSignalSpy aboutToInsertSpy(&treeModel, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)));
+        QSignalSpy insertSpy(&treeModel, SIGNAL(rowsInserted(QModelIndex,int,int)));
 
         ModelUtils::create(&source, insertedStructure, sourceParentPath);
 
@@ -438,6 +439,109 @@ private slots:
         }
     }
 
+    void shouldHandleProjectMoves_data()
+    {
+        QTest::addColumn<ModelStructure>( "sourceStructure" );
+        QTest::addColumn<ModelPath>( "itemToChange" );
+        QTest::addColumn<QString>( "parentUid" );
+        QTest::addColumn<ModelStructure>( "outputStructure" );
+
+        // Base items
+        V inbox(Inbox);
+        C c1(1, 0, "c1");
+        T p1(1, 1, "p1", QString(), "p1", InProgress, ProjectTag);
+        T p2(2, 1, "p2", QString(), "p2", InProgress, ProjectTag);
+        T p3(3, 1, "p3", "p1", "p3", InProgress, ProjectTag);
+        T t1(11, 1, "t1", "p1", "t1");
+        T t2(22, 1, "t2", "p2", "t2");
+        T t3(22, 1, "t3", "p3", "t3");
+        C c2(2, 0, "c2");
+        T p4(4, 2, "p4", QString(), "p4", InProgress, ProjectTag);
+
+        // Create the source structure once and for all
+        ModelStructure sourceStructure;
+        sourceStructure << c1
+                        << _+t1
+                        << _+t2
+                        << _+t3
+                        << _+p1
+                        << _+p2
+                        << _+p3
+                        << c2
+                        << _+p4;
+
+        ModelPath itemToChange = c1 % p2;
+
+        QString parentUid = "p1";
+
+        ModelStructure outputStructure;
+        outputStructure << inbox
+                        << c1
+                        << _+p1
+                        << __+t1
+                        << __+p3
+                        << ___+t3
+                        << __+p2
+                        << ___+t2
+                        << c2
+                        << _+p4;
+
+        QTest::newRow( "root project moved under another project of the same collection" )
+            << sourceStructure << itemToChange
+            << parentUid << outputStructure;
+
+
+        itemToChange = c1 % p3;
+        parentUid = QString();
+        outputStructure.clear();
+        outputStructure << inbox
+                        << c1
+                        << _+p1
+                        << __+t1
+                        << _+p2
+                        << __+t2
+                        << _+p3
+                        << __+t3
+                        << c2
+                        << _+p4;
+
+        QTest::newRow( "sub-project moved as root in the same collection" )
+            << sourceStructure << itemToChange
+            << parentUid << outputStructure;
+
+    }
+
+    void shouldHandleProjectMoves()
+    {
+        //GIVEN
+        QFETCH(ModelStructure, sourceStructure);
+
+        //Source model
+        QStandardItemModel source;
+        ModelUtils::create(&source, sourceStructure);
+
+        //create treeModel
+        TodoTreeModel treeModel;
+        ModelTest t1(&treeModel);
+
+        treeModel.setSourceModel(&source);
+
+        //WHEN
+        QFETCH(ModelPath, itemToChange);
+        QModelIndex index = ModelUtils::locateItem(&source, itemToChange);
+
+        QFETCH(QString, parentUid);
+        source.setData(index, parentUid, Zanshin::ParentUidRole);
+
+        //THEN
+        QFETCH(ModelStructure, outputStructure);
+        QStandardItemModel output;
+        ModelUtils::create(&output, outputStructure);
+
+        QCOMPARE(treeModel, output);
+    }
+
+
     void shouldKeepSourceOrder_data()
     {
         QTest::addColumn<ModelStructure>( "sourceStructure" );
@@ -631,6 +735,100 @@ private slots:
 
         QCOMPARE(todoTreeModel.flags(index), flags);
     }
+
+    void shouldReactToSourceRowRemovalsWithMetadata_data()
+    {
+        QTest::addColumn<ModelStructure>( "sourceStructure" );
+        QTest::addColumn<ModelPath::List>( "itemsToRemove" );
+        QTest::addColumn<ModelStructure>( "outputStructure" );
+
+        // Base items
+        V inbox(Inbox);
+        C c1(1, 0, "c1");
+        C c2(2, 0, "c2");
+        T t1(3, 1, "t1", QString(), "t1", InProgress, ProjectTag);
+        T t2(4, 1, "t2", "t1", "t2");
+        T t3(5, 2, "t3", QString(), "t3", InProgress, ProjectTag);
+        T t4(6, 2, "t4", QString(), "t4", InProgress, ProjectTag);
+        T t5(6, 2, "t5", "", "t5");
+        T t6(6, 2, "t6", "t5", "t6");
+
+        // Create the source structure once and for all
+        ModelStructure sourceStructure;
+        sourceStructure << c1
+                        << _+t1
+                        << _+t2
+                        << c2
+                        << _+t3
+                        << _+t4;
+
+
+        ModelPath::List itemsToRemove;
+        itemsToRemove << c1;
+
+        ModelStructure outputStructure;
+        outputStructure << inbox
+                        << c2
+                        << _+t3
+                        << _+t4;
+
+
+        QTest::newRow( "delete collection" ) << sourceStructure << itemsToRemove << outputStructure;
+
+        sourceStructure.clear();
+        sourceStructure << c1
+                        << _+t5
+                        << _+t6;
+
+        itemsToRemove.clear();
+        itemsToRemove << c1 % t6;
+
+        outputStructure.clear();
+        outputStructure << inbox
+                        << _+t5
+                        << c1;
+
+        QTest::newRow( "Root Project without tag should return to inbox when it looses its children" )
+                                            << sourceStructure << itemsToRemove << outputStructure;
+    }
+
+    void shouldReactToSourceRowRemovalsWithMetadata()
+    {
+        //GIVEN
+        QFETCH(ModelStructure, sourceStructure);
+
+        //Source model
+        QStandardItemModel source;
+        StandardModelBuilderBehavior behavior;
+        behavior.setMetadataCreationEnabled(false);
+        ModelUtils::create(&source, sourceStructure, ModelPath(), &behavior);
+
+        //create metadataModel
+        TodoMetadataModel metadataModel;
+        ModelTest t1(&metadataModel);
+
+        metadataModel.setSourceModel(&source);
+
+        //create treeModel
+        TodoTreeModel treeModel;
+        ModelTest t2(&treeModel);
+
+        treeModel.setSourceModel(&metadataModel);
+
+        //WHEN
+        QFETCH(ModelPath::List, itemsToRemove);
+
+        // destroy the item selected
+        ModelUtils::destroy(&source, itemsToRemove);
+
+        //THEN
+        QFETCH(ModelStructure, outputStructure);
+        QStandardItemModel output;
+        ModelUtils::create(&output, outputStructure);
+
+        QCOMPARE(treeModel, output);
+    }
+
 };
 
 QTEST_KDEMAIN(TodoTreeModelSpec, GUI)
